'-----------------------------------------------------------------------
' <copyright file="Command.vb" company="Microsoft Corporation">
'     Copyright (c) Doug Holland, Microsoft Corporation. All rights reserved.
' </copyright>
'-----------------------------------------------------------------------

Imports System.Windows.Input
Imports System.Threading
Imports System

Namespace MsdnMagazine.HelloWorld.ViewModel
    ''' <summary>
    ''' Provides an implementation of the <see cref="I:System.Windows.Input.ICommand"/> interface to enable commanding.
    ''' </summary>
    Public Class Command
        Implements ICommand

        ''' <summary>
        ''' A predicate to determine whether the command can execute.
        ''' </summary>
        Private ReadOnly canExecutetxt As Predicate(Of Object)

        ''' <summary>
        ''' An action to execute.
        ''' </summary>
        Private ReadOnly execute_Renamed As Action

        ''' <summary>
        ''' The event handler for the <see cref="E:Command.CanExecuteChanged"/> event.
        ''' </summary>
        Private Event canexecuteval As EventHandler Implements ICommand.CanExecuteChanged

        ''' <summary>
        ''' Initializes a new instance of the <see cref="T:Command"/> class.
        ''' </summary>
        ''' <param name="execute">An action to execute.</param>
        Public Sub New(ByVal execute As Action)
            Me.New(execute, Nothing)
        End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="T:Command"/> class.
        ''' </summary>
        ''' <param name="execute">An action to execute.</param>
        ''' <param name="canExecute">A predicate to determine whether the command can execute.</param>
        Public Sub New(ByVal execute As Action, ByVal canExecute As Predicate(Of Object))
            Me.execute_Renamed = execute
            Me.canExecutetxt = canExecute
        End Sub

        ''' <summary>
        ''' Occurs when changes occur that affect whethercanExecute_Renamed or not the command should execute. 
        ''' </summary>
        Public Custom Event CanExecuteChanged As EventHandler
            AddHandler(ByVal value As EventHandler)
                AddHandler Me.canexecuteval, value
            End AddHandler

            RemoveHandler(ByVal value As EventHandler)
                RemoveHandler Me.canexecuteval, value
            End RemoveHandler
            RaiseEvent(ByVal sender As Object, ByVal e As EventArgs)

            End RaiseEvent
        End Event

        ''' <summary>
        ''' Defines the method that determines whether the command can execute in its current state.
        ''' </summary>
        ''' <param name="parameter">Data used by the command. If the command does not require data to be passed, this object can be set to <c>null</c>.</param>
        ''' <returns><c>true</c> if this command can be executed; otherwise, <c>false</c>.</returns>
        Public Function CanExecute(ByVal parameter As Object) As Boolean Implements ICommand.CanExecute
            If Me.canExecutetxt Is Nothing Then
                Return True
            End If

            Return Me.CanExecute(parameter)
        End Function

        ''' <summary>
        ''' Defines the method to be called when the command is invoked.
        ''' </summary>
        ''' <param name="parameter">Data used by the command. If the command does not require data to be passed, this object can be set to <c>null</c>.</param>
        Public Sub Execute(ByVal parameter As Object) Implements ICommand.Execute
            Me.Execute(parameter)
        End Sub

        ''' <summary>
        ''' Raises the <see cref="E:Command.CanExecuteChanged"/> event.
        ''' </summary>
        ''' <param name="e">The <see cref="T:EventArgs"/> instance containing the event data.</param>
        Protected Overridable Sub OnCanExecuteChanged(ByVal e As EventArgs)
            RaiseEvent CanExecuteChanged(Me, e)
        End Sub
    End Class
End Namespace
